/*
 * main.c
 */
#include <asm/io.h>
#include <asm/system.h>
#include <display.h>
#include <irq.h>
#include <traps.h>
#include <keyboard.h>
#include <timer.h>
#include <syscall.h>
#include <kernel.h>
#include <floppy.h>
#include <string.h>

volatile int command;

int main(void)
{
	int cmd,i;
	unsigned char buff[512];
	unsigned char *p = (char *)0x500;

	display_init();
	irq_init();
	traps_init();
	timer_init();
	syscall_init();
	keyboard_init();
	floppy_init();
	sti();

	/*
	 *     
	 *    .
	 */
	for(;;)
	{
		while( !( cmd = command) );

		switch( cmd )
		{
			case 1:
				read_sect(0,buff);
				for( i=0; i < 512; i++ )
				{
					if( buff[i] != p[i] )
					{
						if( (i == 0xfd) || (i == 0x105) )
							continue;
						else
							break;
					}
				}

				if( i != 512 )
					printk("Read sector ERROR: i=%d, b[i]=%x, p[i]=%x\n",i,buff[i],p[i]);
				else puts("Read sector OK\n");

				printk(" st0 = %Xh; st1 = %Xh; st2 = %Xh;"
						"c = %Xh; h = %Xh; r = %Xh; n = %Xh;\n",
						buff[0],buff[1],buff[2],buff[3],buff[4],buff[5],buff[6]);
				printk(" p0 = %Xh; p1 = %Xh; p2 = %Xh; p3 = %Xh;"
						"p4 = %Xh; p5 = %Xh; p6 = %Xh;\n",
						p[0],p[1],p[2],p[3],p[4],p[5],p[6]);
				break;

			case 2:
				read_sect(2000,buff);
				printk("%c %c %c %c %c\n",buff[0],buff[1],buff[2],buff[3],buff[4]);
				break;

			case 3:
				printk(" dir = 0x%x\n",inb(0x3f7));
//				test_sect(buff);
//				printk(" st0 = 0x%x; pcn = 0x%x\n",buff[0],buff[1]);
				break;

			case 4:
				memset(buff,0,512);
				strcpy(buff,"test string");
				if (write_sect(1999,buff))
					puts("write error\n");
				else
					puts("write ok\n");
				break;

			case 5:
				format_sect(1);
				break;

			case 6:
				{
					int i;
					for (i=0; i < 81; i++)
						format_sect(i);
				}
				break;

			case 7:
			case 8:
			case 9:
				break;

			case 10:
				printk(" 0x3f4 = %Xh\n",inb(0x3f4));
				break;
		}

		command = 0;
	}

	return 0;
}
