#include <asm/io.h>
#include <asm/system.h>
#include <irq.h>
#include <display.h>
#include <timer.h>
#include <kernel.h>

#define SIZE_TIMERS       4

struct timers
{
	char flags;
	int id, count;
	timer_func func;
};

extern void irq_timer(void);

static struct jobs_timers
{
	char flags;
	struct timers tm[SIZE_TIMERS];
} tmr;

void do_timer(unsigned int *ad)
{
	int i, rez=0;

	outb(0x20,0x20);

	if( !tmr.flags )
		return;

	for( i=0; i < SIZE_TIMERS; i++ )
	{
		if( tmr.tm[i].flags )
		{
			rez = 1;
			if( !--tmr.tm[i].count )
			{
				tmr.tm[i].flags = 0;
				(*tmr.tm[i].func)(tmr.tm[i].id);
			}
		}
	}

	if( !rez )
		tmr.flags = 0;
}

void timer_init(void)
{
	set_irq(0,irq_timer);
	puts("Timer init OK...\n");
}

int set_timer(int id, int count, timer_func func)
{
	int i, flags, rez=-1;

	flags = get_flags();
	cli();
	{
		for( i=0; i < SIZE_TIMERS; i++ )
		{
			if( !tmr.tm[i].flags )
			{
				tmr.tm[i].id = id;
				tmr.tm[i].count = count;
				tmr.tm[i].func = func;
				tmr.tm[i].flags = 1;
				if( !tmr.flags ) tmr.flags = 1;

				rez = 0;
				break;
			}
		}
	}
	set_flags(flags);

	return rez;
}

int reset_timer(int id, int count)
{
	int i,flags,rez=-1;

	flags = get_flags();
	cli();
	{
		for( i=0; i < SIZE_TIMERS; i++ )
		{
			if( tmr.tm[i].flags && (tmr.tm[i].id == id) )
			{
				tmr.tm[i].count = count;
				rez = 0;
			}
		}
	}
	set_flags(flags);

	return rez;
}

int kill_timer(int id)
{
	int i,flags,rez=-1;

	flags = get_flags();
	cli();
	{
		for( i=0; i < SIZE_TIMERS; i++ )
		{
			if( tmr.tm[i].flags && (tmr.tm[i].id == id) )
			{
				tmr.tm[i].flags = 0;
				rez = 0;
			}
		}
	}
	set_flags(flags);

	return rez;
}
